# /* $Id: //depot/tools/ngs/aggrSpaceChecker/rsh/aggrSpaceCheck.pl#1 $ */

use 5.6.0;
use strict;
use warnings;
use Getopt::Long;
use Pod::Usage;
use File::Temp ();
use IO::Handle;
use File::Basename qw(fileparse);

# Define constants
use constant  VERSION => "1.0.0"; 
use constant  BLOCK_SIZE_BYTES => (1024 * 4); # 4K 
use constant  MAX_NVRAM_SIZE_BYTES => (1024 * 1024 * 1024 * 4); #4G
use constant  SPACE_TAX_VAL => (0.5 /100);     # .5 %
use constant  SPACE_TAX_PFS_VAL => (0.2 /100); # .2 %
use constant  TIMER_SECS => 10;
use constant  VERSION_TEXT => "\naggrSpaceCheck V%s " . 
    "Copyright (c) 2008 NetApp \n";
use constant  WARNING_TEXT => "\nSome volumes may lose guarantee if this ".
    "volume (%s) was resized beyond (%s) " .
    "anytime in the past.\n";
use constant  RESULT_BAD_TEXT => "\nAggregate %s requires %s for volume metadata; " .
    "%s is available." .
    "\nYou must free up %s more space in aggregate %s before upgrading to " .
    "\nData ONTAP 7.3 or later.\n";
use constant RESULT_TEXT => "\nAggregate %s requires %s for volume metadata; " .
    "%s is available." .
    "\nAggregate %s has enough free space for you to upgrade to " . 
    "\nData ONTAP 7.3 or later.\n";

# Global variables with in this file
#PuserName - user argument for the username to be used for connecting to filer
#PfilerName - user argument for the filername to connect to
#config - hash to store config information, like which cmd to be used based on the OS
#verbose - user argument on verbosity
#result - hash to store free space for each aggregates in the system
#errorMsg - to store errors encountered in any step
#execError - flag to denote that there was error while executing cmd. In Linux this is
#            used to cleanup any sub processes.

my($PuserName, $PfilerName, $config, 
   $verbose, $result, $errorMsg, $execErr) = 
    ('root', undef, {}, 0, {}, 0, 0);


# Variables for generating POD in Windows
my($fileName, $path, $podFile);

# Design
#=======
# After setting the configuration - determining which OS and the cmds to be issued,
# first store the free space for each aggregate.Then for each aggregate, get the
# volumes inside that aggregate and do the processing - getting vol type, size, container blocks, etc.
# Based on thos values, calculate space tax and keep adding them for all volumes within a
# aggregate. After computing the aggregate space tax, compare this with aggregate free space. If there
# is not much free space as the required space tax, issue warning to user that additional space is
# required for upgrade to 7.3.x. Otherwise, show that there is enough free space required 
# for 7.3.x upgrade.

# Data ONTAP Commands used by this tool
#* df -A 
# (To determine aggregate space)
# *  vol options <vol name> 
#    (To get volume options - snap_mirrored, fs_size_fixed)
# *  vol status <vol_name> 
#    (To get vol status - offline/online, pfs/flex volume type)
# *  aggr status -v 
#    (To find out each aggregate name and to calculate the space tax)
# *  priv set diag; stats show volume <vol name>; vol size <volume_name>;  
#    (To get the number of container blocks and vol size)

#### Start of functions ####

# Identify the OS and see whether it is supported
# Set the appropriate cmds to be used based on the
# OS to be used.
# Params passed - None

sub setConfig {
    
    my ($remCmd, $options);
    $PuserName = 'root' unless $PuserName;
    
    unless ($^O =~ /MSWin32|linux/i) {
	
	$errorMsg = "Unsupported OS $^O. Only Windows and Linux are supported\n";
	return -1;
    
    }
    
    $remCmd = "rsh";
    if ($^O =~ /MSWin32/i) {
	$options = "$PfilerName -l $PuserName ";
    }

    elsif ($^O =~ /linux/i) {
	$options = "-l $PuserName $PfilerName ";
    }
    
    # Store the cmd to be used.
    $config->{'remCmd'} = $remCmd . " " . $options;
    
    #Add options to redirect std err 
    if ($^O =~ /MSWin32/i) {
	$config->{'remCmdOpts'} = "2>NUL";
    }elsif ($^O =~ /linux/i) {
	$config->{'remCmdOpts'} = "2>/dev/null";
    }
    
    return 0;
}

# By issuing the below cmd, retrieve all the aggregates in the system
# Also for each identified one, retrieve the volumes present in it.
# Params passed - None
# aggr status -v"
#or-183*> aggr status -v
#           Aggr State           Status            Options
#           vol0 online          raid4, trad       root, diskroot, nosnap=off, 
#                                parity uninit'd!  raidtype=raid4, raidsize=8, 
#                                                  ignore_inconsistent=off, 
#                                                  snapmirrored=off, 
#                                                  resyncsnaptime=60, 
#                                                  fs_size_fixed=off, 
#                                                  lost_write_protect=on
#                Volumes: <N/A> 
#                Plex /vol0/plex0: online, normal, active
#                   RAID group /vol0/plex0/rg0: normal
#
#           aggr1 online          raid4, aggr       nosnap=off, raidtype=raid4, 
#                                 parity uninit'd!  raidsize=8, 
#                                                  ignore_inconsistent=off, 
#                                                  snapmirrored=off, 
#                                                  resyncsnaptime=60, 
#                                                  fs_size_fixed=off, 
#                                                  snapshot_autodelete=on, 
#                                                  lost_write_protect=on#
#
#                Volumes: aggr1_vol1, aggr1_vol2, aggr1_vol3
#
#                Plex /aggr1/plex0: online, normal, active
#                    RAID group /aggr1/plex0/rg0: normal

sub getAggrs {
    
    my ($ontapCmd, $loop, $aggr, $vols, $allVols,
	$lastAggr, $cmdOp, $rdyFlag, $aggrStatus);
    
    $ontapCmd = "aggr status -v";
    
    if (($cmdOp = runCmd($ontapCmd)) == -1){
	return -1;
    }
    
    foreach (@$cmdOp){ 
	
	# (text pattern) vol0 online          raid4, trad       root, diskroot, nosnap=off,
	($aggr, $aggrStatus) = /(\w+)\s+(online|offline)\s+.*/i;
	if ($aggr) {
	    
	    $lastAggr = $aggr;
	    $lastAggr =~ s/\s+//g;
	    $vols = undef;
	    $rdyFlag = 0;
	    if ($aggrStatus =~ /offline/i){
		
		$errorMsg = "Aggregate $aggr is offline. ".
		    "Cannot proceed further.\n";
		return -1;

	    }
	    next;
	    
	}
	# Only if vols has not been parsed yet
	unless ($vols){
	    
	    # (text pattern) Volumes: aggr1_vol1, aggr1_vol2, aggr1_vol3
	    ($vols) = /Volumes:(.*)/i;
	    next unless $vols;
	
	    if ($vols =~ /<N\/A>|<none>/i){
	    
		$errorMsg = "No volumes found in aggregate $lastAggr." . 
		    "\nSkipping space calculation.\n";

		# Can go ahead with IC upgrade
		print "\n" . $errorMsg . "\n";
		next;
		#return 0;
		
	    }
	    else {
		$rdyFlag = 1;
	    }
	       
	    unless ($lastAggr){
	    
		$errorMsg = "Got volumes ($vols)  with out getting Aggregate name first";
		return -1;
		
	    }
	}
	
	# process each volume
	if (/Plex/ && $rdyFlag){
	    
	    if (processVols($vols, $lastAggr) == -1){
		return -1;
	    }
	
	}
	else {
	    next if /^\s+$|Volumes:/i;
	    $vols .= $_;
	}
	
    } #End of while loop
    
    return 0;
}

# For each aggregate and the volumes inside it,
# calculate the fee space, space tax and display to
# end user, whether the appregate can be upgraded to 
# ONTAP 7.3 or later.
# Parameters passed - 
#    list of volume names - string, 
#    aggr name - string

sub processVols {
    
    my ($aggr, $vols, $vol, $contSize
	,$flexVol,$pfsVol, $spaceTax);
	
    $vols = shift;
    $aggr = shift;
    
    $vols =~ s/\s+//g;
    print "For Aggregate $aggr volumes are " . 
	$vols . "\n" if $verbose >= 1;
    
    foreach $vol (split(/,/, $vols)){
	
	if ($vol !~ /\w+/){
	    next;
	}
	
	($flexVol, $pfsVol) = chkVolStatus($vol);
	
	# got errors
	if ($flexVol == -1) {
	    return -1;
	}
	    
	# Do calculation if flexible volume
	elsif ($flexVol) {
	    
	    $contSize = getVolSize($vol);
	    if ($contSize == -1) {
		return -1;
	    }
		
	    if ($pfsVol){
		$spaceTax += ($contSize * SPACE_TAX_PFS_VAL);
	    }
	    else {
		$spaceTax += ($contSize * SPACE_TAX_VAL) ;
	    }
	}

	# not flex volume
	else {
	    print "Volume $vol is not a flexible volume. Skipping calculation"
		if $verbose >= 1;
	}
    } # End of for loop
    

    
    if ($spaceTax) { 
	
	if (calSize($spaceTax, $aggr)){
	    return -1;
	}
	
    }
    else {
	    
	$errorMsg = "Could not calculate space for aggregate $aggr";
	print "\n" . $errorMsg . "\n";
	return -1;
    }
    
    return 0;
}

# For a volume name, identify the 
# volume size as well as the container blocks for that vol.
# Also retrieve vol options and get the values for
# snap_mirrored =on || fs_size_fixed=on
# Based on options and container blocks,
# get the adjusted vol size.
# Params passed
# vol name - string

sub getVolSize {

    #sample output
    #vol size is got from the cmd = "priv set diag; stats show volume:<vol name>"
    #volume:vol0:wv_fsinfo_blks_total:23528400

    my ($ontapCmd, $cmdOp, $vol, $contSize, $contBlocks
	, $nominalSize, $nominalSizeStr, $metric, $adjustedContSize 
	,$snapMirr, $fs_size_fixed);
    
    $vol = shift;
    $ontapCmd = "priv set diag; stats show volume:$vol; vol size $vol;";
    
    if (($cmdOp = runCmd($ontapCmd)) == -1){
	return -1;
    }

    foreach (@$cmdOp){ 
	
	#(text pattern) volume:vol0:wv_fsinfo_blks_total:23528400
	($contBlocks) = /volume:${vol}.*wv_fsinfo_blks_total:([^\t\n\r\f]+).*/i
	    unless defined $contBlocks;
	
	#(text pattern) vol size: Flexible volume 'aggr1_vol1' has size 26773320k.
	($nominalSize, $metric) = /has size (\d+)(\w)/i
	    unless (defined $nominalSize && defined $metric);
	
	last if (defined $contBlocks 
		 && defined $nominalSize 
		 && defined $metric);
	
    }
    
    unless (defined $contBlocks && 
	    defined $nominalSize && 
	    defined $metric){
	
	$errorMsg = "Could not retrieve values for Container File System Blocks count "
	    ."or volume size for volume $vol \n";
	return -1;
	
    }
	    
    $contSize = $contBlocks * BLOCK_SIZE_BYTES;

    print "$vol has container blocks of $contBlocks. " .
	"Container size  is " . getSizeStr($contSize) ."\n" 
	if $verbose >= 1;
    
    # determine vol size
    $nominalSize *= getSizeFactor($metric);
    $nominalSizeStr = getSizeStr($nominalSize);
    
    print "$vol has nominal size of ${nominalSizeStr} \n" 
	if $verbose >= 1;
    
    # Check snap_mirrored =on || fs_size_fixed=on
    # If they are one, adjustments need to be made
    # to the calculated container file size
    ($snapMirr, $fs_size_fixed) = chkVolOptions($vol);
    
    if ($snapMirr == -1) {
	return -1;
    }
    
    elsif (($snapMirr) || ($fs_size_fixed)){
	
	$adjustedContSize = getAdjustedContSize($nominalSize);
	if ($adjustedContSize > $contSize){
	    $contSize = $adjustedContSize;
	}
	
	print "Adjusted Container file system size is  $contSize\n"
	    if $verbose >= 1;
	
	# Issue warning to user
	printf "\n" . WARNING_TEXT . "\n", $vol ,$nominalSizeStr;
    
    }
    
    return $contSize;
}

# Retrieve vol status (offline or online),
# and vol type - pfs vol or flex vol or trad vol
# Params passed
# vol name - string
# Sample o/p
#or-183*> vol status  aggr1_vol1
#         Volume State           Status            Options
#     aggr1_vol1 online          raid4, flex       
#                                parity uninit'd!  
#                Containing aggregate: 'aggr1'

sub chkVolStatus {

    my ($ontapCmd, $cmdOp, $vol, $vsize
	,$volType, $pfsVol, $volStatus);

    $vol = shift;
    
    if ($vol !~ /\w+/){
	$errorMsg = "chkVolStatus: Passed an invalid volume name ($vol)";
	return wantarray ? (-1, -1, -1) : -1;
    }

    $ontapCmd = "vol status $vol";
    
    if (($cmdOp = runCmd($ontapCmd)) == -1){
	return wantarray ? (-1, -1, -1) : -1;
    }

    foreach (@$cmdOp){ 
	
	#(text pattern)  aggr1_vol1 online          raid4, flex   
	($volStatus) = /(\boffline\b)/i unless $volStatus;
	($volType) = /(\bflex\b)/ unless $volType;
	($pfsVol)  = /(\bflexcache\b)/ unless $pfsVol;
	last if $volType && $pfsVol;
    }
    
    
    if ($volType){
	$volType = 1;
    }
    else {
	$volType = 0;
    }

    if ($pfsVol){
	$pfsVol = 1;
    }
    else {
	$pfsVol = 0;
    }
    
    if ($volStatus && $volType) {
	
	$errorMsg = "Flexible volume $vol is offline. " .
	    "Cannot proceed further.\n";
	return wantarray ? (-1, -1, -1) : -1;
    
    }

    return wantarray ? ($volType, $pfsVol) : "$volType,$pfsVol";
}

# Retrieve options for a volume and get the
# values for snapmirrored and fs_size_fixed
# sample o/p
# Param passed
# vol name - string
#or-183*> vol options aggr1_vol1
#nosnap=off, nosnapdir=off, minra=off, no_atime_update=off, nvfail=off, 
#ignore_inconsistent=off, snapmirrored=off, create_ucode=off, 
#convert_ucode=off, maxdirsize=167690, schedsnapname=ordinal, 
#fs_size_fixed=off, compression=off, guarantee=volume, svo_enable=off, 
#svo_checksum=off, svo_allow_rman=off, svo_reject_errors=off, 
#no_i2p=off, fractional_reserve=100, extent=off, try_first=volume_grow, 
#snapshot_clone_dependency=off

sub chkVolOptions {

    my ($ontapCmd, $cmdOp, $vol
	,$snapMirr, $fsSize);

    $vol = shift;
    $vol =~ s/\s+//g;
    
    if ($vol !~ /\w+/){
	$errorMsg = "ChkVolOptions: Passed an invalid volume name ($vol)";
	return wantarray ? (-1, -1) : -1;
    }
    
    $ontapCmd = "vol options $vol";
    
    if (($cmdOp = runCmd($ontapCmd)) == -1){
	return wantarray ? (-1, -1) : -1;
    }
    
    foreach (@$cmdOp){ 
	
	#ignore_inconsistent=off, snapmirrored=off, create_ucode=off
	($snapMirr) = /snapmirrored=(\w+)/ unless $snapMirr;
	
	#fs_size_fixed=off, compression=off, guarantee=volume, svo_enable=off,
	($fsSize)   = /fs_size_fixed=(\w+)/ unless $fsSize;

	last if $snapMirr && $fsSize;
	
    }
    
    if ($snapMirr && ($snapMirr =~ /on/i)){
	$snapMirr = 1;
    }
    else {
	$snapMirr = 0;
    }

    if ($fsSize && ($fsSize =~ /on/i)){
	$fsSize = 1;
    }
    else {
	$fsSize = 0;
    }
    
    return wantarray ? ($snapMirr, $fsSize) : "$snapMirr,$fsSize";
}

# For the vol size, make adjustments.
# See BURT 253271 for details of algorithm
# params passed
# vol size - integer
sub getAdjustedContSize {

    my ($nominalSize, $adjustedContSize);

    $nominalSize = shift;
    $nominalSize =~ s/\s+//g;
    
    # Do validation
    if ($nominalSize !~ /\d+/){
	return wantarray ? (0) : "0";
    }
     
    if (2 * $nominalSize < MAX_NVRAM_SIZE_BYTES) {
	$adjustedContSize = 2 * $nominalSize;
    }
    elsif (1.1 * $nominalSize < MAX_NVRAM_SIZE_BYTES) {
	$adjustedContSize = MAX_NVRAM_SIZE_BYTES;
    }
    else {
	$adjustedContSize = 1.1 * $nominalSize;
    }
    
    return wantarray ? ($adjustedContSize) : $adjustedContSize;
}

# For a given size, convert it to easily readable string
# Passed params -
# - size - integer
sub getSizeStr {

    my ($size, $sizeStr);
    
    $size = shift;
    
    # Bytes
    if ($size/1024 < 1 ) {
	$sizeStr = sprintf("%d bytes", $size);
    }

    # Kilo bytes
    elsif ($size/(1024**2) < 1 ) {
	$sizeStr = sprintf("%4.2fKB", $size/1024);
    }

    # Mega bytes
    elsif ($size/(1024**3) < 1 ) {
	$sizeStr = sprintf("%4.2fMB", $size/(1024**2));
    }

    # Giga bytes
    elsif ($size/(1024**4) < 1 ) {
	$sizeStr = sprintf("%4.2fGB", $size/(1024**3));
    }
    
    # Tera bytes
    elsif ($size/(1024**5) < 1 ) {
	$sizeStr = sprintf("%4.2fTB", $size/(1024**4));
    }

    # Peta bytes
    elsif ($size/(1024**6) < 1 ) {
	$sizeStr = sprintf("%4.2fPB", $size/(1024**5));
    }
    
    else {
	$sizeStr = sprintf("%d bytes", $size);
    }

    return wantarray ? ($sizeStr) : $sizeStr;
}

# For a given size as KB or MB, or other formsm
# make it to the smallest factor - bytes
# Para passed
# metric - string.
sub getSizeFactor {
    
    my ($metric, $factor);
    
    $metric = shift;
    
    # kilo bytes
    if ($metric =~ /k/i){
	$factor = 1024;
    }
    
    # Mega bytes
    elsif ($metric =~ /m/i){
	$factor = 1024 ** 2;
    }
    
    # Giga bytes
    elsif ($metric =~ /g/i){
	$factor = 1024 ** 3;
    }

    # Tera bytes
    elsif ($metric =~ /t/i){
	$factor = 1024 ** 4;
    }
    
    # Peta bytes
    elsif ($metric =~ /p/i){
	$factor = 1024 ** 5;
    }
    
    else {
	$factor = 1;
    }
    
    return wantarray ? ($factor) : $factor;
}

# For the connected system, issue the below cmd and thus retrieve
# aggregates prsenet and their free space.
# STore the result in %result, which is used later to determine
# there is enough space tax
# Params passed - None
# sample o/p
#or-183*> df -A
#Aggregate               kbytes       used      avail capacity  
#aggr3                 29749752    2119860   27629892       7%  
#aggr3/.snapshot        1565776      11488    1554288       1%  
#vol0                  25052424     776524   24275900       3%  
#vol0/.snapshot         6263104      14380    6248724       0%  
#aggr2                 29749752   29741248       8504     100%  
#aggr2/.snapshot        1565776      27324    1538452       2%  
#aggr1                 29749752   29749752          0     100%  
#aggr1/.snapshot        1565776      18092    1547684       1%  

sub getAggrFreeSpace {
    
    my ($ontapCmd, $cmdOp); 
    my ($spacer, $aggrName, $aggrSize, $aggrUsed, $aggrAvail);

    $errorMsg = undef;
    
    # df -A"    
    $ontapCmd = "df -A";
    
    if (($cmdOp = runCmd($ontapCmd)) == -1){
	return -1;
    }
    
    foreach (@$cmdOp){ 
	
	$aggrName = $aggrAvail = undef;
	
	#(text pattern) aggr3                 29749752    2119860   27629892       7% 
	($spacer, $aggrName, $aggrSize, $aggrUsed, $aggrAvail) = 
	    m#^(\s+)?([^\t\n\r\f/]+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+).*#o;

	# Store free space of each aggregate
	if (defined $aggrName && defined $aggrAvail) {

	    $aggrName =~ s/\s+//g;
	    $aggrAvail =~ s/\s+//g;
	    $result->{$aggrName} = $aggrAvail * getSizeFactor("k");
	    
	    print "Aggregate $aggrName has free space of " . 
		getSizeStr($result->{$aggrName}) . "\n"
		if $verbose >= 1;
	    
	}
    
    }
    unless (keys %{$result}){
	$errorMsg = "Could not get any aggregates in this filer";
	return -1;
    }
    
    return 0;
}

# Given the space tax for an aggregate
# compare with the free space available
# and display to user whether the aggregate is
# ready for upgrade to 7.3.x
# Params passed
# - aggregate name - String
# - space tax - String
sub calSize {

    my ($spaceTax, $aggr, $aggrFreeSpace, $diff);
    
    $spaceTax = shift;
    $aggr = shift;
    $aggrFreeSpace = $result->{$aggr};
    
    unless (defined $aggrFreeSpace){
	
	$errorMsg = "Could not retrieve free space for aggregate $aggr";
	return -1;
    
    }
    
    $diff = $aggrFreeSpace - $spaceTax;
    # Does not have needed space
    if ($diff < 0) {

	printf(RESULT_BAD_TEXT
	       ,$aggr
	       ,getSizeStr($spaceTax)
	       ,getSizeStr($aggrFreeSpace)
	       ,getSizeStr($spaceTax-$aggrFreeSpace)
	       ,$aggr
	       );
	
    }
    else {
	
	printf(RESULT_TEXT
	       ,$aggr
	       ,getSizeStr($spaceTax)
	       ,getSizeStr($aggrFreeSpace)
	       ,$aggr
	       );
    
    }
    return 0;

}

# Run the remote cmd from here.
# Would set a timer, so long running jobs would be stopped 
# from running for ever.
# Params passed -
# command to be run - string
sub runCmd {

    my ($cmdArgs, $cmd, $result);
    
    $cmdArgs = shift;
    
    $cmd = $config->{'remCmd'} . " " 
	. "\"$cmdArgs\"" 
	. " " 
	. $config->{'remCmdOpts'}; 
    
    print "Cmd is $cmd \n" 
	if $verbose >= 1;
    
    # Do inside eval block, for timer requirement
    eval {
    
	local $SIG{ALRM} = sub { die "Timer alarm\n" };
	
        # Set timer
	alarm TIMER_SECS;
	
	@$result = `$cmd`;
	
        # clear timer
	alarm 0;
	
    };
    
    # Errors caught
    if ($@) {
	
	$errorMsg = "Error in executing cmd $cmd. $!" 
	    unless $@ eq "Timer alarm\n";
	$errorMsg = "Command $cmd timed out. $!"
	    if $@ eq "Timer alarm\n";	
	$execErr  = 1;
	return -1;
	
    }
    
    #results got
    return $result;
    
}

##############
# Main program
##############

printf VERSION_TEXT, VERSION;

($fileName, $path) = fileparse($0);

if (defined $fileName && $fileName =~ /\.exe$/) {
    $podFile = $path . "README.txt";
}
else {
    $podFile = $0;
}

# Retrieve the cmd line arguments
GetOptions('help|?' => 
	   sub {pod2usage(-verbose=>99,
			  -sections=>"NAME|SYNOPSIS|DESCRIPTION|OPTIONS" . 
			  "|PREREQUISITES|INSTALLATION|EXAMPLES|FILES|HISTORY",
			  -input=>$podFile
			  )},
	   'filer=s' => \$PfilerName,
	   'user:s' => \$PuserName,
	   'verbose:i' => \$verbose
	   );

# Invoke usage if needed
pod2usage({-exitval=>1,
	   -verbose=>0,
	   -input=>$podFile}) unless $PfilerName;


# Set the remote cmd to be used
if (setConfig()){
    
    print "\nCould not set tool configuration. $errorMsg\n";
    exit(1);
    
}


# Populate free space
if (getAggrFreeSpace()){
    
    print "\nCould not retrieve Aggregate free space. $errorMsg\n";
    exit(1);

}

# Determine space tax availability
if (getAggrs()){
    
    print "\n$errorMsg\n";
    exit(1);

}

exit(0);

# Cleanup any spawned processes
# by killing the process group in Linux
END {
    
    kill -9, getpgrp()
	if $execErr && ($^O =~ /linux/i);
    
}

__END__

=head1 NAME 

aggrSpaceCheck - a utility that identifies the amount of free space required
on each aggregate before upgrading to Data ONTAP 7.3 or later.

=head1 SYNOPSIS

aggrSpaceCheck [options] -filer <system_name>

  Options:
    -user     Username for connecting to storage system
    -verbose  Specify 1 to display debugging me
    -help     Displays help for options
    
=head1 DESCRIPTION

When installed on a Microsoft Windows or Linux client system,
aggrSpaceCheck connects to a storage system using the RSH 
protocol and checks whether there is enough free space to permit
upgrading to Data ONTAP 7.3 or later.

After connecting to storage system, it executes the following Data ONTAP 
commands:

df -A
  determines aggregate space

vol options <vol_name>
  gets volume options; values are snap_mirrored, fs_size_fixed

vol status <vol_name>
  gets vol status; values are offline/online, pfs/flex volume type

aggr status -v
  discovers each aggregate name and calculates the space requirement

priv set diag; stats show volume <vol name>; vol size <volume_name>;
  gets the number of container blocks and vol size

=head1 OPTIONS

=over 8

=item B<-user>

    The user name for connecting to a storage system.
    The default behavior is to use the root user,
    unless a different user is specified.
    This user name need to be in the storage system's
    "Administrators" group.
    Example: -username operator

=item B<-help>

    Displays usage and examples.

=item B<-verbose>
    
    For debug purpose only. Verbose 0 reports minimal debug
        information. 1 has a larger debug information
        Example: -verbose 0
                 -verbose 1

=back

=head1 PREREQUISITES

For running Data ONTAP commands non-interactively,
RSH connectivity must be configured on the
storage system.

To configure RSH, see the Data ONTAP 7.3 Network Management Guide.

The user name for connecting to a storage system must
be in the Storage system's "Administrators" group.

Aggregates and Volumes within a Storage system needs to
be "online"

Supported platforms: Microsoft Windows XP, Microsoft Windows 2000,
Windows Vista and Linux.

    Microsoft Windows environments
    -------------------------------
    The executable aggrSpaceCheck.exe for Microsoft Windows 
    does not require Perl.

    The RSH program must be in current users "path" environment variable.

    Linux environments
    ------------------
    To run aggrSpaceCheck.pl, a standard installation of
    Perl 5.6.0 or later is required.

    In Linux environments, the rsh program needs to be in current
    user's "path" environment variable.


=head1 INSTALLATION

To install aggrSpaceCheck, download the aggrSpaceCheck.zip
file and unzip it in any folder.

=head1 EXAMPLES

To connect to the storage system "server1" with user name "root"
enter the following command:
    aggrSpaceCheck.exe -filer server1

To connect to the system "server1" with user name "operator" 
enter:
    aggrSpaceCheck.exe -user operator -filer server1
    
    Sample script output
    --------------------

    [samv@ngst101-dev aggrSpace]$ perl aggrSpaceCheck.pl -filer or-183

    aggrSpaceCheck V1.02 Copyright (c) 2008 NetApp 

    No volumes found in aggregate vol0.
    Skipping space calculation.

    Aggregate aggr1 requires 169.71MB for volume metadata; 
    0 bytes is available. You must free up 169.71MB more space 
    in aggregate aggr1 before upgrading to Data ONTAP 7.3 or later.

    Aggregate aggr2 requires 170.95MB for volume metadata; 
    8.30MB is available. You must free up 162.65MB more 
    space in aggregate aggr2 before upgrading to 
    Data ONTAP 7.3 or later.

    Aggregate aggr3 requires 20.99MB for volume metadata; 
    26.35GB is available.
    Aggregate aggr3 has enough free space  for you to upgrade to 
    Data ONTAP 7.3 or later.

=head1 FILES

    aggrSpaceCheck.pl
        perl script, runs as:
        perl aggrSpaceCheck.pl -filer <system_name>

    aggrSpaceCheck.exe
        Microsoft Windows executable file
    

=head1 HISTORY

Version : 1.0.0

=cut


